@@ -9,16 +9,16 @@ use warnings;
 use Encode qw(decode_utf8);
 use LWP::UserAgent;
 use Readonly;
-use Text::Iconv;
+use URI;
 use Web::Scraper;
 
 # Constants.
 Readonly::Scalar our $MAINTAINER => 'Michal Spacek <skim@cpan.org>';
-Readonly::Scalar my $VALENTINSKA_CZ => 'http://valentinska.cz/';
-Readonly::Scalar my $VALENTINSKA_CZ_ACTION1 => 'index.php?hledani=Vyhledej&hltex=';
+Readonly::Scalar my $VALENTINSKA_CZ => 'http://www.valentinska.cz/';
+Readonly::Scalar my $VALENTINSKA_CZ_ACTION1 => 'index.php?route=product/search&search=';
 
 # Version.
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
 # Setup.
 sub native_setup_search {
@@ -26,16 +26,28 @@ sub native_setup_search {
 	$self->{'_def'} = scraper {
 
 		# Get list of books.
-		process '//table[@class="ProductTable"]/tr/td',
+		process '//div[@class="product-inner clearfix"]',
 			'books[]' => scraper {
 
-			process '//h2/a', 'title' => 'TEXT';
-			process '//h2/a', 'url' => '@href';
-			process '//img', 'cover_url' => '@src';
-			process '//p[@class="AuthorName"]',
+			process '//div[@class="wrap-infor"]/div[@class="name"]/a',
+				'title' => 'TEXT';
+			process '//div[@class="wrap-infor"]/div[@class="name"]/a',
+				'url' => ['@href', sub {
+					my $url = shift;
+					my $uri = URI->new($url);
+					my %query = $uri->query_form;
+					if (exists $query{'search'}) {
+						delete $query{'search'};
+					}
+					$uri->query_form(\%query);
+					return $uri->as_string;
+				}];
+			process '//div[@class="image"]/a/img',
+				'image' => '@src';
+			process '//div[@class="wrap-infor"]/div[@class="author"]',
 				'author' => 'TEXT';
-			process '//p[@class="BookPrice"]/span',
-				'price' => 'TEXT';
+			process '//div[@class="wrap-infor"]/div[@class="price"]/br/preceding-sibling::text()',
+				'price' => ['TEXT', sub { s/^\s*(.*?)\s*$/$1/; }];
 			return;
 		};
 		return;
@@ -49,30 +61,24 @@ sub native_retrieve_some {
 	my $self = shift;
 
 	# Query.
-	my $i1 = Text::Iconv->new('utf-8', 'windows-1250');
-	my $query = $i1->convert(decode_utf8($self->{'_query'}));
+	my $query = decode_utf8($self->{'_query'});
 
 	# Get content.
 	my $ua = LWP::UserAgent->new(
-		'agent' => "WWW::Search::KacurCZ/$VERSION",
-	);
-	my $response = $ua->get($VALENTINSKA_CZ.
-		$VALENTINSKA_CZ_ACTION1.$query,
+		'agent' => "WWW::Search::ValentinskaCZ/$VERSION",
 	);
+	my $query_url = $VALENTINSKA_CZ.$VALENTINSKA_CZ_ACTION1.$query;
+	my $response = $ua->get($query_url);
 
 	# Process.
 	if ($response->is_success) {
-		my $i2 = Text::Iconv->new('windows-1250', 'utf-8');
-		my $content = $i2->convert($response->content);
+		my $content = $response->content;
 
 		# Get books structure.
 		my $books_hr = $self->{'_def'}->scrape($content);
 
 		# Process each book.
 		foreach my $book_hr (@{$books_hr->{'books'}}) {
-			_fix_url($book_hr, 'url');
-			_fix_url($book_hr, 'cover_url');
-			$self->_remove_tr($book_hr, 'title');
 			push @{$self->{'cache'}}, $book_hr;
 		}
 	}
@@ -80,26 +86,6 @@ sub native_retrieve_some {
 	return;
 }
 
-# Fix URL to absolute path.
-sub _fix_url {
-	my ($book_hr, $url) = @_;
-	if (exists $book_hr->{$url}) {
-		$book_hr->{$url} = $VALENTINSKA_CZ.$book_hr->{$url};
-	}
-	return;
-}
-
-# Remove trailing whitespace.
-sub _remove_tr {
-	my ($self, $book_hr, $key) = @_;
-	if (! exists $book_hr->{$key}) {
-		return;
-	}
-	$book_hr->{$key} =~ s/^\s+//gms;
-	$book_hr->{$key} =~ s/\s+$//gms;
-	return;
-}
-
 1;
 
 __END__
@@ -167,24 +153,35 @@ WWW::Search::ValentinskaCZ - Class for searching http://valentinska.cz .
 
  # Output with 'Čapek' argument like:
  # \ {
- #     author      "Norbert F. Čapek",
- #     cover_url   "http://valentinska.cz/images/0506fdc6967e087e3aa666301b530864.jpg",
- #     price       "80 Kč",
- #     title       " Myšlenky",
- #     url         "http://valentinska.cz/index.php?lang=&idvyrb=380039&akc=detail"
+ #     author   "Larbaud, Valery; obálka: J. Čapek",
+ #     image    "http://www.valentinska.cz/image/cache/data/valentinska/book_144061_1-1024x1024.jpg",
+ #     price    "450Kč",
+ #     title    "A. O. Barnbooth. Jeho důvěrný deník",
+ #     url      "http://www.valentinska.cz/144061-a-o-barnbooth-jeho-duverny-denik"
  # }
 
 =head1 DEPENDENCIES
 
-L<HTTP::Cookies>,
+L<Encode>,
 L<LWP::UserAgent>,
 L<Readonly>,
+L<URI>,
 L<Web::Scraper>,
 L<WWW::Search>.
 
 =head1 SEE ALSO
 
-L<WWW::Search>.
+=over
+
+=item L<WWW::Search>
+
+Virtual base class for WWW searches
+
+=item L<Task::WWW::Search::Antiquarian::Czech>
+
+Install the WWW::Search modules for Czech antiquarian bookstores.
+
+=back
 
 =head1 REPOSITORY
 
@@ -198,10 +195,11 @@ L<http://skim.cz>
 
 =head1 LICENSE AND COPYRIGHT
 
-BSD license.
+ © Michal Špaček 2014-2015
+ BSD 2-Clause License
 
 =head1 VERSION
 
-0.02
+0.03
 
 =cut
